'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var PropTypes = _interopDefault(require('prop-types'));
var ReactDOM = _interopDefault(require('react-dom'));
var server = require('react-dom/server');

var canUseDom = (function () {
  return typeof window !== 'undefined';
});

var capitalize = (function (word) {
  return word.replace(/^./, function (letter) {
    return letter.toUpperCase();
  });
});

var hasOwnProperty = (function (object, prop) {
  return Object.prototype.hasOwnProperty.call(object, prop);
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var HAS_RESOLVED = 'ReactResolver.HAS_RESOLVED';
var IS_CLIENT = 'ReactResolver.IS_CLIENT';

var Resolver = function (_React$Component) {
  inherits(Resolver, _React$Component);

  function Resolver(props, context) {
    classCallCheck(this, Resolver);

    // Internal tracking variables
    var _this = possibleConstructorReturn(this, (Resolver.__proto__ || Object.getPrototypeOf(Resolver)).call(this, props, context));

    _this.unMounted = true;
    _this[HAS_RESOLVED] = false;
    _this[IS_CLIENT] = false;

    _this.state = _this.computeState(_this.props, {
      pending: {},
      resolved: {}
    });

    if (_this.isPending(_this.state)) {
      _this.resolve(_this.state);
      _this[HAS_RESOLVED] = false;
    } else {
      _this[HAS_RESOLVED] = true;
    }
    return _this;
  }

  createClass(Resolver, [{
    key: 'cached',
    value: function cached(resolve) {
      var _props = this.props,
          props = _props.props,
          cache = _props.cache;


      if (cache || !canUseDom()) {
        if (hasOwnProperty(props, resolve)) {
          return props[resolve];
        } else if (this.context.resolver) {
          return this.context.resolver.cached(resolve);
        }
      }

      return null;
    }
  }, {
    key: 'componentWillMount',
    value: function componentWillMount() {
      this.unMounted = false;
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      this[IS_CLIENT] = true;
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.unMounted = true;
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      var _computeState = this.computeState(nextProps, this.state),
          pending = _computeState.pending,
          resolved = _computeState.resolved;

      // Next state will resolve async props again, but update existing sync props


      var nextState = {
        pending: pending,
        resolved: resolved
      };

      this.setAtomicState(nextState);
    }
  }, {
    key: 'computeState',
    value: function computeState(thisProps, state) {
      var _this2 = this;

      var resolve = thisProps.resolve;

      var nextState = state;

      Object.keys(resolve).forEach(function (name) {
        var cached = _this2.cached(name);

        if (!hasOwnProperty(state.resolved, name) && !hasOwnProperty(state.pending, name) && !_this2.isValidCache(cached)) {
          nextState.pending[name] = resolve[name];
        } else if (cached) {
          nextState.resolved[name] = true;
        }
      });

      return nextState;
    }
  }, {
    key: 'getChildContext',
    value: function getChildContext() {
      return { resolver: this };
    }
  }, {
    key: 'isValidCache',
    value: function isValidCache(cache) {
      return cache !== null && typeof cache !== 'undefined' && ((typeof cache === 'undefined' ? 'undefined' : _typeof(cache)) !== 'object' || Object.keys(cache).length > 0) && (!Array.isArray(cache) || cache.length > 0);
    }
  }, {
    key: 'isPending',
    value: function isPending() {
      var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.state;

      return Object.keys(state.pending).length > 0;
    }
  }, {
    key: 'isParentPending',
    value: function isParentPending() {
      var resolver = this.context.resolver;


      if (resolver) {
        return resolver.isPending() || resolver.isParentPending();
      }

      return false;
    }
  }, {
    key: 'onResolve',
    value: function onResolve(state) {
      if (this.props.onResolve) {
        return this.props.onResolve(state);
      } else if (this.context.resolver) {
        return this.context.resolver.onResolve(state);
      } else {
        return state;
      }
    }
  }, {
    key: 'render',
    value: function render() {
      // Avoid rendering until ready
      if (!this[HAS_RESOLVED]) {
        return false;
      }

      // If render is called again (e.g. hot-reloading), re-resolve
      if (this.isPending(this.state)) {
        this.resolve(this.state);
      }

      // Both those props provided by parent & dynamically resolved
      return this.props.children(_extends({}, this.props.props));
    }
  }, {
    key: 'resolve',
    value: function resolve(state) {
      var _this3 = this;

      var props = this.props.props;


      var pending = Object.keys(state.pending).map(function (name) {
        var func = state.pending[name];

        return { name: name, func: func };
      });

      var promises = pending.map(function (_ref) {
        var func = _ref.func;
        return func(props);
      });

      var resolving = Promise.all(promises).then(function (values) {
        return values.reduce(function (resolved, value, i) {
          resolved[pending[i].name] = true;

          return resolved;
        }, {});
      });

      // Resolve listeners get the current resolved
      resolving = this.onResolve(resolving);

      // Update current component (on client)
      resolving.then(function (resolved) {
        _this3[HAS_RESOLVED] = true;

        if (!_this3[IS_CLIENT]) {
          return false;
        }

        var nextState = {
          pending: {},
          resolved: _extends({}, state.resolved, resolved)
        };

        _this3.setAtomicState(nextState);
      });
    }
  }, {
    key: 'shouldComponentUpdate',
    value: function shouldComponentUpdate(nextProps, nextState) {
      // Prevent updating when parent is changing values
      if (this.isParentPending()) {
        return false;
      }

      // Prevent rendering until pending values are resolved
      if (this.isPending(nextState)) {
        return false;
      }

      // Update if we have resolved successfully
      return this[HAS_RESOLVED];
    }
  }, {
    key: 'setAtomicState',
    value: function setAtomicState(nextState) {
      if (this.unMounted) {
        return;
      }

      this.setState(nextState);
    }
  }]);
  return Resolver;
}(React.Component);

Resolver.childContextTypes = {
  resolver: PropTypes.object
};
Resolver.contextTypes = {
  resolver: PropTypes.object
};
Resolver.defaultProps = {
  props: {},
  resolve: {}
};
Resolver.displayName = 'Resolver';
Resolver.propTypes = {
  children: PropTypes.func.isRequired,
  props: PropTypes.object,
  resolve: PropTypes.object
};

Resolver.renderClient = function (render, node) {
  ReactDOM.hydrate(React.createElement(
    Resolver,
    null,
    render
  ), node);
};

Resolver.renderServer = function (_render) {
  var initialData = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var queue = [];

  server.renderToStaticMarkup(React.createElement(
    Resolver,
    { onResolve: function onResolve(promise) {
        queue.push(promise);

        return Promise.resolve(true);
      } },
    _render
  ));

  return Promise.all(queue).then(function (results) {
    var formatResults = {};
    results.forEach(function (item) {
      return Object.keys(item).forEach(function (key) {
        formatResults[key] = item[key];
      });
    });

    var data = _extends({}, initialData, formatResults);

    if (Object.keys(initialData).length < Object.keys(data).length) {
      return Resolver.renderServer(_render, data);
    }

    var Resolved = function (_React$Component2) {
      inherits(Resolved, _React$Component2);

      function Resolved() {
        classCallCheck(this, Resolved);
        return possibleConstructorReturn(this, (Resolved.__proto__ || Object.getPrototypeOf(Resolved)).apply(this, arguments));
      }

      createClass(Resolved, [{
        key: 'render',
        value: function render() {
          return React.createElement(
            Resolver,
            null,
            _render
          );
        }
      }]);
      return Resolved;
    }(React.Component);

    Resolved.displayName = 'Resolved';


    return Resolved;
  });
};

var resolve = (function (prop, promise) {
  var cache = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;


  var asyncProps = (typeof prop === 'undefined' ? 'undefined' : _typeof(prop)) === 'object' ? prop : defineProperty({}, prop, promise);
  var asyncNames = Object.keys(asyncProps).map(capitalize).join('');
  var cacheEnabled = (typeof prop === 'undefined' ? 'undefined' : _typeof(prop)) === 'object' && typeof promise === 'boolean' ? promise : cache;

  return function (Component) {
    var _class, _temp;

    return _temp = _class = function (_React$Component) {
      inherits(_class, _React$Component);

      function _class() {
        classCallCheck(this, _class);
        return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
      }

      createClass(_class, [{
        key: 'render',
        value: function render() {
          return React.createElement(
            Resolver,
            { props: this.props, resolve: asyncProps, cache: cacheEnabled },
            function (resolved) {
              return React.createElement(Component, resolved);
            }
          );
        }
      }]);
      return _class;
    }(React.Component), _class.displayName = asyncNames + 'Resolver', _temp;
  };
});

var client = (function (prop, Loader) {
  var promise = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;


  var loadProps = typeof prop === 'string' ? [prop] : prop;
  var names = loadProps.map(capitalize).join('');

  return function (Component) {
    var _class, _temp;

    return _temp = _class = function (_React$Component) {
      inherits(_class, _React$Component);

      function _class(props, context) {
        classCallCheck(this, _class);

        var _this = possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).call(this, props, context));

        _this.enqueue = function (promise) {
          _this.queue.push(promise);

          return promise;
        };

        _this.isLoaded = function () {
          var loaded = true;

          loadProps.forEach(function (prop) {
            if (!hasOwnProperty(_this.props, prop) || !_this.props[prop]) {
              loaded = false;
            }
          });

          return loaded;
        };

        _this.queue = [];
        _this.state = {
          bypass: !canUseDom() && !promise || process.env.NODE_ENV === 'test',
          loaded: _this.isLoaded(props)
        };
        return _this;
      }

      createClass(_class, [{
        key: 'componentDidMount',
        value: function componentDidMount() {
          if (promise) {
            promise(this.props);
          }
        }
      }, {
        key: 'render',
        value: function render() {
          var _this2 = this;

          var bypass = this.state.bypass;


          if (bypass || this.isLoaded()) {
            return React.createElement(Component, this.props);
          }

          return React.createElement(
            React.Fragment,
            null,
            React.createElement(Loader, null),
            !promise && React.createElement(
              'div',
              { style: { display: 'none' } },
              React.createElement(
                Resolver,
                { onResolve: this.enqueue },
                function (resolved) {
                  return React.createElement(Component, _extends({}, _this2.props, resolved));
                }
              )
            )
          );
        }
      }]);
      return _class;
    }(React.Component), _class.displayName = names + 'ClientResolver', _class.contextTypes = {
      resolver: PropTypes.instanceOf(Resolver)
    }, _temp;
  };
});

exports.resolve = resolve;
exports.client = client;
exports.Resolver = Resolver;
//# sourceMappingURL=react-esc-resolver.development.js.map
